////////////////////////////////////////////////////////////////////////////////
// 
// Adjacency.hh 
//
//    produced: 21/05/04 cw
// last change: 21/05/04 cw
//
// This program is free software; Please read COPYING
//  (GNU General Public Licence) for details
//
////////////////////////////////////////////////////////////////////////////////
#ifndef ADJACENCY_HH
#define ADJACENCY_HH

#include <assert.h>
#include <cmath>

#include "Array.hh"
#include "IntegerVector.hh"

#include "CommandlineOptions.hh"

static const parameter_type NULLVERT = 0;

typedef Array<IntegerVector> adjacency_data;

class Adjacency : public adjacency_data {
public:
  // constructors:
  inline Adjacency();
  inline Adjacency(const Adjacency&);
  inline Adjacency(const size_type, 
		   const size_type, 
		   const parameter_type& = 0);
  // accessors:
  inline const size_type get(const size_type vx_no, const size_type ngbr_no) const;
  // operations in place:
  // operations out of place:
  // boolean expressions:
};

// constructors:
inline Adjacency::Adjacency() : 
  adjacency_data() {
}
inline Adjacency::Adjacency(const Adjacency& adjacency) : 
  adjacency_data(adjacency) {
}

inline Adjacency::Adjacency(const size_type init_rowdim, 
			    const size_type init_coldim, 
			    const parameter_type& init_entry) : 
  adjacency_data(init_coldim) {
  for (size_type i = 0; i < init_coldim; ++i) {
    (*this)[i] = IntegerVector(init_rowdim, init_entry);
  }
}

inline const size_type Adjacency::get(const size_type vx_no, const size_type ngbr_no) const {
  assert(vx_no<=maxindex());
  if (ngbr_no<=((*this)[vx_no]).maxindex())
    return (*this)[vx_no][ngbr_no];
  return NULLVERT;
}

#endif

// eof Adjacency.hh
