////////////////////////////////////////////////////////////////////////////////
// 
// CommandlineOptions.cc
//
//    produced: 04 Oct 1999 jr
// last change: 04 Oct 1999 jr
//
// This program is free software; Please read COPYING
//  (GNU General Public Licence) for details
//  this file was written by Joerg Rambau for TOPCOM
//
////////////////////////////////////////////////////////////////////////////////

#include <iostream>

#include "CommandlineOptions.hh"

bool      CommandlineOptions::_debug                   = false;
bool      CommandlineOptions::_mute                    = false;
bool      CommandlineOptions::_positive                = false;
bool      CommandlineOptions::_shortoutput             = false;
bool      CommandlineOptions::_longoutput              = false;
bool      CommandlineOptions::_verbose                 = false;
int       CommandlineOptions::_proc                    = 1;

void CommandlineOptions::init(const int argc, const char** argv) {
  for (int i = 1; i < argc; ++i) {

    // print usage message:
    if ((strcmp(argv[i], "--help") == 0) || (strcmp(argv[i], "-h") == 0)) {
      std::cout << "Usage:" << std::endl;
      std::cout << argv[0] << "[options]" << std::endl;
      std::cout << "where the input is read from stdin" << std::endl;
      std::cout << "and the output is written to stdout." << std::endl;
      std::cout << std::endl;

      std::cout << "options concerning shell output:" << std::endl;
      std::cout << "-h or --help         : print this help message" << std::endl;
      std::cout << "-c                   : for minkSum, write also the normal cone of vertices of the solution. Slower." << std::endl;
      std::cout << "-s                   : for minkSum, write only vertices coordinates of the solution." << std::endl;
      std::cout << "-n N                 : for minkSumFork and minkSumForkGrid, compute the sum in parallel using N processes" << std::endl;
      std::cout << "-d                   : write debug information to stderr." << std::endl;
      std::cout << "-m                   : mute. Don't write anything to stderr" << std::endl;
      std::cout << "-p                   : for minkSum, ignore vertices who don't have a positive maximizer"
		<< std::endl;
      std::cout << std::endl;

      exit(0);
    }

    // output options:
    if (strcmp(argv[i], "-d") == 0) {
      _debug = true;
    }
    if (strcmp(argv[i], "-m") == 0) {
      _mute = true;
    }
    if (strcmp(argv[i], "-s") == 0) {
      _shortoutput = true;
    }
    if (strcmp(argv[i], "-c") == 0) {
      _longoutput = true;
    }
    if (strcmp(argv[i], "-v") == 0) {
      _verbose = true;
    }
    if (strcmp(argv[i], "-n") == 0) {
      if (i+1<argc)
	sscanf(argv[i+1],"%d",&_proc);
    }

    // options for accelerating algorithms:
    if (strcmp(argv[i], "-p") == 0) {
      _positive = true;
    }
  }
}

// eof CommandlineOptions.cc
