////////////////////////////////////////////////////////////////////////////////
// 
// IntegerMatrix.hh 
//
//    produced: 21/05/04 cw
// last change: 21/05/04 cw
//
// This program is free software; Please read COPYING
//  (GNU General Public Licence) for details
//
////////////////////////////////////////////////////////////////////////////////
#ifndef INTEGERMATRIX_HH
#define INTEGERMATRIX_HH

#include <assert.h>
#include <cmath>

#include "Array.hh"
#include "Adjacency.hh"
#include "IntegerVector.hh"

#include "CommandlineOptions.hh"

typedef Array<IntegerVector> integermatrix_data;

class IntegerMatrix : public integermatrix_data {
public:
  // constructors:
  inline IntegerMatrix();
  inline IntegerMatrix(const IntegerMatrix&);
  inline IntegerMatrix(const size_type, 
		       const size_type, 
		       const parameter_type& = 0);
  // accessors:
  inline const size_type get(const size_type vx_no, const size_type ngbr_no) const;
  // operations in place:
  // operations out of place:
  // boolean expressions:
};

// constructors:
inline IntegerMatrix::IntegerMatrix() : 
  integermatrix_data() {
}
inline IntegerMatrix::IntegerMatrix(const IntegerMatrix& integermatrix) : 
  integermatrix_data(integermatrix) {
}

inline IntegerMatrix::IntegerMatrix(const size_type init_rowdim, 
			    const size_type init_coldim, 
			    const parameter_type& init_entry) : 
  integermatrix_data(init_coldim) {
  for (size_type i = 0; i < init_coldim; ++i) {
    (*this)[i] = IntegerVector(init_rowdim, init_entry);
  }
}

inline const size_type IntegerMatrix::get(const size_type vx_no, const size_type ngbr_no) const {
  assert(vx_no<=maxindex());
  if (ngbr_no<=((*this)[vx_no-1]).maxindex())
    return (*this)[vx_no-1][ngbr_no-1];
  return NULLVERT;
}

#endif

// eof IntegerMatrix.hh
