////////////////////////////////////////////////////////////////////////////////
// 
// LPinter.hh 
//
//    produced: 2004/04/28 cw
//
// This program is free software; Please read COPYING
//  (GNU General Public Licence) for details
//  this file was adapted from the file LPinterface by Joerg Rambau for TOPCOM
//
////////////////////////////////////////////////////////////////////////////////
#ifndef LPINTER_HH
#define LPINTER_HH

#include "setoper.h"
#include "Rational.h"

// the following preprocessor magic is necessary because the word `Rational'
// is used in an enum type in the cddlib:

#include "cdd.h"

// we can use three variants of arithmetics in cddlib:

#if defined GMPRATIONAL // GMP Rational
  
inline void dd_bis_set_R(mytype& a, const Rational& b) {
  //  mpq_init(a);
  mpq_set(a, b.get_rep());
}

#elif defined GMPFLOAT // GMP float
  
inline void dd_bis_set_R(mytype& a, const Rational& b) {
  //  mpq_init(a);
  mpq_set(a, double(b));
}

#else // built-in C double 
  
inline void dd_bis_set_R(mytype& a, const Rational& b) {
  a[0] = double(b);
}

#endif

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <string.h>

#include "Matrix.hh"

class LPinter {
private:
  dd_ErrorType     _err;
  dd_LPSolverType  _solver;
  dd_LPPtr         _lpptr;
  dd_LPPtr         _mainlpptr;
  dd_LPSolutionPtr _solptr; 
  dd_rowrange      _m;
  dd_colrange      _n;
  dd_MatrixPtr     _matrixptr;
private:
  LPinter();
public:
  // constructors:
  LPinter(const Matrix&, const Vector&);
  // destructor:
  inline ~LPinter();
  // functions:
  Vector solve();
  bool has_interior_point();
};

inline LPinter::~LPinter() {
  
  // Free allocated spaces:
  //dd_FreeLPSolution(_solptr); // Causes a segmentation fault. No idea why.
  dd_FreeLPData    (_lpptr);
  dd_FreeLPData    (_mainlpptr);
  dd_FreeMatrix    (_matrixptr);
}

#endif

// eof LPinter.hh
