////////////////////////////////////////////////////////////////////////////////
// 
// MatrixList.hh 
//
//    produced: 22/11/05 cw
// last change: 22/11/05 cw
//
////////////////////////////////////////////////////////////////////////////////
#ifndef MATRIXLIST_HH
#define MATRIXLIST_HH

#include <assert.h>

#include "Array.hh"

#include "CommandlineOptions.hh"

#include "Vector.hh"

typedef Array<Matrix> matrixlist_data;

class MatrixList : public matrixlist_data {
public:
  static const char col_delim_char;
public:
  // constructors:
  inline MatrixList();
  inline MatrixList(const MatrixList&);
  inline MatrixList(const Matrix&);
  inline MatrixList(const size_type, const size_type, const size_type, const Field& init_entry = ZERO);
  // destructor:
  inline ~MatrixList();
  // accessors:
  inline const size_type rowdim() const;
  inline const size_type coldim() const;
  inline const size_type poldim() const;
  const        Vector row(const size_type) const;
  inline const Vector& col(const size_type) const;
  // resizing:
  void        row_resize(const size_type, const Field& init_entry = ZERO);
  inline void col_resize(const size_type, const Field& init_entry = ZERO);
  // indexing:
  inline       Field& operator()(const size_type, const size_type);
  inline const Field& operator()(const size_type, const size_type) const;
  // in place operations:
  MatrixList& add(const MatrixList&);
  MatrixList& scale(const Field&);
  MatrixList& augment(const MatrixList&);
  MatrixList& stack(const MatrixList&);
  MatrixList& swap_cols(const size_type, const size_type);
  MatrixList& swap_rows(const size_type, const size_type);
  MatrixList& swap_pols(const size_type, const size_type);
  // out of place operations:
  // iostream:
  std::ostream& pretty_print(std::ostream& ost) const;
};

inline MatrixList::MatrixList() : matrixlist_data() {
}
inline MatrixList::MatrixList(const MatrixList& matrixlist) : matrixlist_data(matrixlist) {
}
inline MatrixList::MatrixList(const Matrix& matrix) : matrixlist_data(1,matrix) {
}
inline MatrixList::MatrixList(const size_type init_rowdim, 
		      const size_type init_coldim, 
		      const size_type init_poldim, 
		      const Field& init_entry) : 
  matrixlist_data(init_poldim) {
  for (size_type i = 0; i < init_poldim; ++i) {
    (*this)[i] = Matrix(init_rowdim, init_coldim, init_entry);
  }
}
inline MatrixList::~MatrixList() {}

inline const size_type MatrixList::rowdim() const {
  assert(maxindex() != 0);
  assert((*this)[0].maxindex() != 0);
  return (*this)[0][0].maxindex();
}
inline const size_type Matrix::coldim() const {
  assert(maxindex() != 0);
  return (*this)[0].maxindex();
}
inline const size_type MatrixList::poldim() const {
  return maxindex();
}

inline const Vector& MatrixList::pol(const size_type index) const {
  return (*this)[index];
}

inline void MatrixList::pol_resize(const size_type new_size, const Field& init_entry) {
  matrixlist_data::resize(new_size, Matrix(rowdim(), coldim(), init_entry));
}

inline Field& MatrixList::operator()(const size_type row, const size_type col) {
  return (*this)[col][row];
}

inline const Field& MatrixList::operator()(const size_type row, const size_type col,pol) const {
  return (*this)[pol][col][row];
}

#endif

// eof MatrixList.hh
