////////////////////////////////////////////////////////////////////////////////
// 
// RSinter.cc
//
//    produced: 2004/10/14 cw
//
// This program is free software; Please read COPYING
//  (GNU General Public Licence) for details
//
////////////////////////////////////////////////////////////////////////////////

#include "RSinter.hh"

// constructors:

RSinter::RSinter(const Matrix& m, const Vector &vo, const Vector &rs)
{
  if (CommandlineOptions::debug()) {
    std::cerr << "building RS matrix ..." << std::endl;
  }

  // note that the coefficient matrix m is transposed for efficiency reasons;
  _m                          = m.coldim(); // number of rows
  _n                          = m.rowdim(); // number of cols
  _matrixptr                  = dd_CreateMatrix(_m, _n);
  _matrixptr->numbtype        = dd_Rational;
  dd_InitializeArow(_n,&_vo);
  dd_InitializeArow(_n,&_rs);

  for (size_type j = 0; j < m.rowdim(); ++j) {

    // build cdd's matrix row by row;
    // recall that the coefficient matrix m is transposed for efficiency reasons;
    // build origin vector too
    dd_bis_set_R(_vo[j], vo[j]);
    // build shooting ray too
    dd_bis_set_R(_rs[j], rs[j]);

    for (size_type i = 0; i < m.coldim(); ++i) {

      // because the first column contains the data for the right hand side 
      // (which is ZERO) we need to shift the column index by one;
      // again, we have to use the coefficient matrix m in a transposed way:
      dd_bis_set_R(_matrixptr->matrix[i][j], m(j, i));
    }
  }
  if (CommandlineOptions::debug()) {
    dd_WriteMatrix(stderr, _matrixptr);
    std::cerr << "... done." << std::endl;
  }
}

// functions:
int RSinter::shoot() {

  return dd_RayShooting(_matrixptr,_vo,_rs);
}

// eof RSinter.cc
