////////////////////////////////////////////////////////////////////////////////
// 
// SumVertex.hh 
//
//    produced: 30/11/06 cw
//
// This program is free software; Please read COPYING
//  (GNU General Public Licence) for details
//  this file was written by Christophe Weibel for POLYLIB
//
// The point of this object is to contain the information relative
// to a vertex of the Minkowski Sum:
//   - vertex coordinates
//   - decomposition in the sum
//   - maximizer, ie vector maximizing the sum on the vertex
//   - incident edges, as vectors oriented towards the vertex
//
////////////////////////////////////////////////////////////////////////////////
#ifndef SUMVERTEX_HH
#define SUMVERTEX_HH


#include "Array.hh"
#include "IntegerVector.hh"
#include "Vector.hh"
#include "VPolytope.hh"

class SumVertex {
private:
  Vector _coord;
  IntegerVector _decomp;
  Vector _max;
  VPolytope _edges;

public:
  // constructors:
  inline SumVertex();
  inline SumVertex(const SumVertex&);
  inline SumVertex(const Vector&,
		   const IntegerVector&,
		   const Vector&,
		   const VPolytope&);

  // accessors:

  inline Vector& coord();
  inline const Vector& coord() const;

  inline IntegerVector& decomposition();
  inline const IntegerVector& decomposition() const;

  inline Vector& maximizer();
  inline const Vector& maximizer() const;

  inline VPolytope& edges();
  inline const VPolytope& edges() const;

  // iostream:
  inline std::ostream& write(std::ostream&) const;
  friend std::ostream& operator<<(std::ostream& ost, const SumVertex& a);
};

inline SumVertex::SumVertex() {  
}

inline SumVertex::SumVertex(const SumVertex& arg) {
  _coord = arg.coord();
  _decomp = arg.decomposition();
  _max = arg.maximizer();
  _edges = arg.edges();
}

inline SumVertex::SumVertex(const Vector& coord,
			    const IntegerVector& decomp,
			    const Vector& max,
			    const VPolytope& edges) {
  _coord = coord;
  _decomp = decomp;
  _max = max;
  _edges = edges;
}

inline Vector& SumVertex::coord() {
  return _coord;
}

inline const Vector& SumVertex::coord() const {
  return _coord;  
}

inline IntegerVector& SumVertex::decomposition() {
  return _decomp;
}

inline const IntegerVector& SumVertex::decomposition() const {
  return _decomp;  
}

inline Vector& SumVertex::maximizer() {
  return _max;
}

inline const Vector& SumVertex::maximizer() const {
  return _max;  
}

inline VPolytope& SumVertex::edges() {
  return _edges;
}

inline const VPolytope& SumVertex::edges() const {
  return _edges;  
}

// iostream:
inline std::ostream& SumVertex::write(std::ostream& ost) const {
  Vector outmax = Vector(_max[1]);
  for (int i=2; i<_max.maxindex(); ++i) {
    outmax.append(_max[i]);
  }
  if (_edges.no())
    return ost << _decomp << " : " << _coord << " : "  << outmax << " : " << _edges << std::endl;
  else 
    return ost << _decomp << " : " << _coord << " : "  << outmax <<  std::endl;
}


inline std::ostream& operator<<(std::ostream& ost, const SumVertex& a) {
  return a.write(ost);
}

#endif

// eof SumVertex.hh
