////////////////////////////////////////////////////////////////////////////////
// 
// VPolytopeList.hh 
//
//    produced: 22/11/05 cw
// last change: 22/11/05 cw
//
////////////////////////////////////////////////////////////////////////////////
#ifndef VPOLYTOPELIST_HH
#define VPOLYTOPELIST_HH

#include <assert.h>

#include "setoper.h"
#include "Rational.h"

#include "cdd.h"

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <string.h>
#include "Vector.hh"
#include "Matrix.hh"
#include "VPolytope.hh"
#include "SumVertexList.hh"
#include "LPinter.hh"
#include "RSinter.hh"

#include "CommandlineOptions.hh"

// (*this)[pol][col][row], points are in row ((*this)[0][0] = extreme vector)

const int MINKSUM_CHECK_LEVEL = 4;

typedef Array<VPolytope> vpolytopelist_data;

class VPolytopeList : public vpolytopelist_data {
private:
  Vector obj; // Maximizer of the first vertex, used to orient edges
  IntegerVector vertex; // root of the tree
  Vector vertexMax; // maximizer of vertex, stored for efficiency

public:
  Vector maximizer(const IntegerVector);
  IntegerVector neighbour(const IntegerVector, const int, const int);
  bool adj(const  IntegerVector cur_vtx, const int poly, const int ngbr_no);
  IntegerVector parent(IntegerVector, Vector);
  Vector output(const IntegerVector sol) const;
  void realOutput(const IntegerVector sol) const;
  void realOutput(const IntegerVector sol, std::ostream&) const;
  void print0(const Vector sol) const;
  void print0(const Vector sol, std::ostream&) const;
  void print(const IntegerVector sol) const;
  void print(const IntegerVector sol, std::ostream&) const;
  // constructors:
  inline VPolytopeList();
  inline VPolytopeList(const VPolytopeList&);
  inline VPolytopeList(const VPolytope&);
  inline VPolytopeList(const size_type, const size_type, const size_type, const Field& init_entry = ZERO);

  // destructor:
  inline ~VPolytopeList();

  // accessors:
  inline const size_type numPol() const;
  inline const VPolytope& pol(const size_type) const;
  inline const Vector& getObj() const;
  inline IntegerVector& getBest();
  inline std::istream&  readObj(std::istream&);

  // adjacency
  inline const size_type faj(const size_type& pol_no, const size_type& vtx_no,const size_type& ngbr_no) {
    assert(pol_no>0);
    assert(vtx_no>0);
    assert(ngbr_no>0);
    return (*this)[pol_no].faj(vtx_no, ngbr_no);
  }
  inline Vector fej(const size_type& pol_no, const size_type& vtx_no,const size_type& ngbr_no) {
    assert(pol_no>0);    
    return (*this)[pol_no].fej(vtx_no, ngbr_no);
  }
  inline Vector fej0(const size_type& pol_no, const size_type& vtx_no,const size_type& ngbr_no) {
    assert(pol_no>0);    
    return (*this)[pol_no].fej0(vtx_no, ngbr_no);    
  }

  // COMPUTATIONS

  // Removes redundancies, that is, computes the convex hull
  VPolytope removeRedundancy();

  // Sets up a Minkowski sum computation
  // Computes ajdacency of polytopes and the first vertex to output
  void incMinkSumSetup();

  // Writes in files the result of the setup
  void outputSetup();

  // Outputs a vertex, and computes the next one if it exists
  // Outputs facets of the normal cone, but is slower
  SumVertex incExploreStep();

  // Outputs a vertex, and computes the next one if it exists
  // Does not output facets of the normal cone
  SumVertex incExploreStepFast();

  // Checks if there is a vertex waiting to be output
  bool hasMoreVertices();

};

inline VPolytopeList::~VPolytopeList() {}

inline VPolytopeList::VPolytopeList() : vpolytopelist_data() {
}
inline VPolytopeList::VPolytopeList(const VPolytopeList& vpolylist) : vpolytopelist_data(vpolylist) {
}
inline VPolytopeList::VPolytopeList(const VPolytope& vpoly) : vpolytopelist_data(1,vpoly) {
}

inline const size_type VPolytopeList::numPol() const {
  return maxindex();
}

inline const Vector& VPolytopeList::getObj() const {
  return obj;
}

inline IntegerVector& VPolytopeList::getBest() {
  return vertex;
}

inline std::istream&  VPolytopeList::readObj(std::istream& ist) {
  return ist >> obj;
}


#endif

// eof VPolytopeList.hh
