/*  This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <assert.h>
#include <fstream>
#include <errno.h>
#include <fcntl.h>
#include "CommandlineOptions.hh"
#include "Array.hh"
#include "VPolytope.hh"
#include "VPolytopeList.hh"
#include "IntegerMatrix.hh"
#include "LPinter.hh"
#include "RSinter.hh"

const int MAX_PROC = 16;
VPolytopeList polylist;
bool first = true;
size_type proc;
int bossProc;
int bossChanges;
int workdone = 0;
int sendBlock[MAX_PROC];
int recvBlock[MAX_PROC];
int send[MAX_PROC];
int recv[MAX_PROC];
size_type proc_num;
char outFile[15];
std::ofstream out;
int messNr = 0;

void explore(const IntegerVector current, Vector currentMax) {

  VPolytope facets = VPolytope();

  for (int j=0; j<polylist.numPol(); ++j)
    for (int i=0;
	 i<polylist[j].ngbrCount(current[j]);
	 ++i) { // loop on leaving edges

      bool adj;
      Vector edge;
      if (CommandlineOptions::longoutput()) {
	// in this case we must compute adj anyway
	adj = polylist.adj(current,j,i);
	if (adj) { // and add the -edge to the list if adj
	  edge = polylist[j].fej0(current[j],i);
	  edge.scale(-1);
	  facets.append(edge);
	}
      }

      if (CommandlineOptions::longoutput() ?
	  adj && inner_product(edge,polylist.getObj())>0 :
	  (inner_product(polylist[j].fej0(current[j],i),polylist.getObj())<0 &&
	   polylist.adj(current,j,i))) { // examine only away from objective
	IntegerVector test = polylist.neighbour(current,j,i);

	Vector testMax = polylist.maximizer(test);
	if (testMax.maxindex()==0) { // No satisfactory maximizer found
	  // actually, this sould only happen if we choose
	  // to forbid maximizers with negative values
	  if (!CommandlineOptions::positive()) {
	    std::cerr << "ERROR : THIS IS A BUG 2" << std::endl;
	  }
	  continue; // don't examine point at all.
	}

	Matrix pbV = Matrix(testMax.maxindex(),0);

	Vector diff = Vector(testMax);
	diff.scale(-1);
	diff.add(polylist.getObj());

	for (int l=0; l<polylist.numPol(); ++l)
	  for (int k=0;
	       k<polylist[l].ngbrCount(test[l]);
	       ++k)
	    if (inner_product(polylist[l].fej0(test[l],k),diff)>0) {
	      Vector toAdd = Vector(polylist[l].fej0(test[l],k));
	      toAdd.scale(-1);
	      pbV.append(toAdd);
	    }

	RSinter RSV = RSinter(pbV,testMax,diff);
	int limit;

	limit = RSV.shoot();

	if (limit < 1) {
	  fprintf(stderr,"ERROR : THIS IS A BUG\n");
	}

	int firstPol = 0;
	int firstNgbr = 0;
	bool cont = true;

	// find out which is the parent for current vertex
	for (int l=0; cont && l<polylist.numPol(); ++l)
	  for (int k=0; cont && k<polylist[l].ngbrCount(test[l]); ++k)
	    if (inner_product(polylist[l].fej0(test[l],k),diff)>0) {
	      if (--limit == 0) {
		cont = false;
		firstPol = l;
		firstNgbr = k;
	      }
	    }

	if (polylist.neighbour(test,firstPol,firstNgbr)==current) { // check parent
	  int freeProc = -1;
	  int res, message;
	  for (int p=0; (freeProc<0) && (p<proc_num); ++p) {
	    if (p == proc) continue;
	    bool keepReading = true;
	    while (keepReading) {
	      keepReading = false;
	      res = read(recv[p],&message,sizeof(message));
	      if (res == -1) {
		assert(errno == EAGAIN);
	      } else {
		assert(res == sizeof(message));
		if (message < bossChanges) {
		  if (CommandlineOptions::verbose()) {
		    ++messNr;
		    std::cerr << (messNr*10+proc+1000000);
		    out << "proc " << proc << " (#" << messNr << "); got an old message from " << p << std::endl;
		  }
		  keepReading = true;
		} else {
		  if (message > bossChanges) {
		    if (CommandlineOptions::verbose()) {
		      ++messNr;
		      std::cerr << (messNr*10+proc+1000000);
		      out << "proc " << proc << " (#" << messNr << "); is suddenly told he is the boss number " << message << " by " << p  << std::endl;
		    }
		    bossChanges = message;
		    bossProc = proc;
		  }
		  freeProc = p;
		}
	      }
	    } // while keepReading
	  } // for p
	  if (freeProc < 0) {
	    explore(test,testMax);
	  } else {
	    int sendWork = -1;
	    if (CommandlineOptions::verbose()) {
	      ++messNr;
	      std::cerr << (messNr*10+proc+1000000);
	      out << "proc " << proc << " (#" << messNr << "); sending work(-1) to " << freeProc << std::endl;
	    }
	    write(sendBlock[freeProc],&sendWork,sizeof(sendWork));
	    for (int l=0;l<polylist.numPol();++l) {
	      write(sendBlock[freeProc],&test[l],sizeof(test[l]));
	    }
	  }
	} // check parent
      } // examine only away from objective
    } // loop on leaving edges

  if (!CommandlineOptions::shortoutput()) {
    polylist.print(current,out);
    out << " : ";
  } else {
    if (!first)
      out << "," << std::endl;
    else
      first = false;
  }
  polylist.realOutput(current,out);
  if (!CommandlineOptions::shortoutput()) {
    out << " : ";
    polylist.print0(currentMax,out);
    if (CommandlineOptions::longoutput()) {
      out << " : ";
      out << facets;
    }
    out << std::endl;
  }
}

int main(const int argc, const char** argv) {
  CommandlineOptions::init(argc, argv);
  proc_num = CommandlineOptions::proc();

  dd_set_global_constants();  // First, this must be called to use cdd

  // Read the polytopes

  if (!polylist.read(std::cin)) {
    fprintf(stderr,"error while reading polytopes\n");
    return 1;
  }

  if (!CommandlineOptions::mute()) {
    fprintf(stderr,"Sum of %d polytopes\n",polylist.numPol());
  }

  polylist.incMinkSumSetup();

  int pipeGridBlock[proc_num][proc_num][2];
  int pipeGrid[proc_num][proc_num][2];

  for (int i=0; i<proc_num; ++i)
    for (int j=0; j<proc_num; ++j) {
      if (i==j) continue;
      assert(pipe(pipeGridBlock[i][j]) >= 0);
      assert(pipe(pipeGrid[i][j]) >= 0);
    }

  proc = 0;
  int pid = 0;
  while ((!pid) && (proc < proc_num-1)) {
    pid = fork();
    assert(pid != -1);
    if (pid == 0) ++proc;
  }


  for (int i=0; i<proc_num; ++i)
    for (int j=0; j<proc_num; ++j) {
      if (i==j) continue;
      if (i==proc) {
	sendBlock[j] = pipeGridBlock[i][j][1];
	send[j] = pipeGrid[i][j][1];
      } else {
	close(pipeGridBlock[i][j][1]);
	close(pipeGrid[i][j][1]);
      }
      if (j==proc) {
	recvBlock[i] = pipeGridBlock[i][j][0];
	recv[i] = pipeGrid[i][j][0];
	fcntl(recv[i],F_SETFL,O_NONBLOCK);
      } else {
	close(pipeGridBlock[i][j][0]);
	close(pipeGrid[i][j][0]);
      }
    }

  sprintf(outFile,"result.%d.out",proc);
  out.open(outFile);

  if (CommandlineOptions::shortoutput()) {
    out << "[";
  }

  if (CommandlineOptions::verbose()) {
    ++messNr;
    std::cerr << (messNr*10+proc+1000000);
    out << "proc " << proc << " (#" << messNr << "); ready to work" << std::endl;
  }

  IntegerVector root = polylist.getBest();
  Vector rootMax = polylist.getObj();
  int res, message;

  int cont = 1;
  int queueEmpty = 0;
  bossProc = 0;
  bossChanges = 0;

  if (proc == bossProc) {
    explore(root,rootMax);
    ++workdone;
  }

  bool dontAskForWork = false;

  while (cont) {
    if (CommandlineOptions::verbose()) {
      ++messNr;
      std::cerr << (messNr*10+proc+1000000);
      out << "proc " << proc << " (#" << messNr << "); starting new loop" << std::endl;
    }
    if (proc == bossProc) {
      bossProc = -1;
      for (int p=0; (bossProc<0) && (p<proc_num); ++p) {
	if (p == proc) continue;

	int res, message;
	bool keepReading = true;
	while (keepReading) {
	  keepReading = false;
	  res = read(recv[p],&message,sizeof(message));
	  if (res == -1) {
	    assert(errno == EAGAIN);
	    // found busy proc
	    if (CommandlineOptions::verbose()) {
	      ++messNr;
	      std::cerr << (messNr*10+proc+1000000);
	      out << "proc " << proc << " (#" << messNr << "); found " << p << " is busy" << std::endl;
	    }
	    bossProc = p;
	  } else {
	    assert(res == sizeof(message));
	    if (message < bossChanges) {
	      if (CommandlineOptions::verbose()) {
		++messNr;
		std::cerr << (messNr*10+proc+1000000);
		out << "proc " << proc << " (#" << messNr << "); boss got an old message from " << p << std::endl;
	      }
	      keepReading = true;
	    } else {
	      if (message > bossChanges) {
		if (CommandlineOptions::verbose()) {
		  ++messNr;
		  std::cerr << (messNr*10+proc+1000000);
		  out << "proc " << proc << " (#" << messNr << "); boss is suddenly told he is the boss number " << message << " by " << p << std::endl;
		}
		bossChanges = message;
	      } // message > bossChanges
	      if (CommandlineOptions::verbose()) {
		++messNr;
		std::cerr << (messNr*10+proc+1000000);
		out << "proc " << proc << " (#" << messNr << "); found " << p << " is asking for work" << std::endl;
	      }
	    }
	  } // found proc asking for work
	} // while keepReading
      } // for p

      ++bossChanges;
      for (int p=0; p<proc_num; ++p) {
	if (p == proc) continue;
	if (CommandlineOptions::verbose()) {
	  ++messNr;
	  std::cerr << (messNr*10+proc+1000000);
	  out << "proc " << proc << " (#" << messNr << "); sending bossChange Nr." << bossChanges << "(" << bossProc << ") to " << p << std::endl;
	}
	write(sendBlock[p],&bossChanges,sizeof(bossChanges));
	write(sendBlock[p],&bossProc,sizeof(bossProc));
      } // for p
      if (CommandlineOptions::verbose()) {
	++messNr;
	std::cerr << (messNr*10+proc+1000000);
	out << "proc " << proc << " (#" << messNr << "); new boss is " << bossProc << std::endl;
      }
      if (bossProc < 0) cont = false;
    } else { // if proc == bossProc

      if (!dontAskForWork) {
	message = bossChanges; // ask for work
	if (CommandlineOptions::verbose()) {
	  ++messNr;
	  std::cerr << (messNr*10+proc+1000000);
	  out << "proc " << proc << " (#" << messNr << "); asking for work to " << bossProc << std::endl;
	}
	write(send[bossProc],&message,sizeof(message));
      } else {
	dontAskForWork = false;
      }
      if (CommandlineOptions::verbose()) {
	++messNr;
	std::cerr << (messNr*10+proc+1000000);
	  out << "proc " << proc << " (#" << messNr << "); waiting" << std::endl;
      }

      res = read(recvBlock[bossProc],&message,sizeof(message));
      assert(res != -1);
      assert(res == sizeof(message));

      if (CommandlineOptions::verbose()) {
	++messNr;
	std::cerr << (messNr*10+proc+1000000);
	out << "proc " << proc << " (#" << messNr << "); read " << message << std::endl;
      }

      if (message == -1) { // signal that bossProc gave work
	IntegerVector work = IntegerVector();
	for (int l=0;l<polylist.numPol();++l) {
	  parameter_type temp;
	  res = read(recvBlock[bossProc],&temp,sizeof(temp));
	  assert(res != -1);
	  assert(res == sizeof(temp));
	  work.append(temp);
	}
	Vector workMax = polylist.maximizer(work);
	explore(work,workMax);
	++workdone;

	if (CommandlineOptions::verbose()) {
	  ++messNr;
	  std::cerr << (messNr*10+proc+1000000);
	  out << "proc " << proc << " (#" << messNr << "); finished job " << workdone << std::endl;
	}
      } else if (message > 0) { // if message == -1 / > 0
	int message2;
	res = read(recvBlock[bossProc],&message2,sizeof(message2));
	assert(res != -1);
	assert(res == sizeof(message2));

	if (message > bossChanges) {
	  bossChanges = message;
	  bossProc = message2;
	  if (bossProc < 0) cont = false;
	  if (CommandlineOptions::verbose()) {
	    ++messNr;
	    std::cerr << (messNr*10+proc+1000000);
	    out << "proc " << proc << " (#" << messNr << "); learns that boss Nr." << message << " is " <<  message2 << std::endl;
	  }
	} else {
	  dontAskForWork = true;
	  if (CommandlineOptions::verbose()) {
	    ++messNr;
	    std::cerr << (messNr*10+proc+1000000);
	    out << "proc " << proc << " (#" << messNr << "); learns that boss Nr." << message << " was " <<  message2 << ", but it is old news" << std::endl;
	  }
	}
	
      } else { // no other messages
	assert(0 == 1); // received an alien message
      } // no other messages
    } // if proc != bossProc
  } // while cont

  //  sleep(2);

  if (CommandlineOptions::verbose()) {
    ++messNr;
    std::cerr << (messNr*10+proc+1000000);
    out << "proc " << proc << " (#" << messNr << "); out after " << workdone << " jobs done" << std::endl;
  }

  for (int p=0; p<proc_num; ++p) {
    if (p == proc) continue;
    close(recvBlock[p]);
    close(sendBlock[p]);
    close(recv[p]);
    close(send[p]);
  }

  if (CommandlineOptions::shortoutput()) {
    out << "]" << std::endl;
  }
  out.close();

  if (CommandlineOptions::verbose()) {
    ++messNr;
    std::cerr << (messNr*10+proc+1000000);
    out << "proc " << proc << " out after " << workdone << " jobs done" << std::endl;
  }
  dd_free_global_constants();  // Nice
  return 0;
}
